//********************************************************
//
// Example Application of BRD_xxx functions for BASEDAC service.
//
// (C) InSys by Dorokhin Andrey Jul, 2006
//
//=********************************************************

#include	<windows.h>
#include	<conio.h>

#include	"brd.h"
#include	"extn_andor.h"
#include	"ctrlstrm.h"
#include	"CtrlBaseDac.h"

#include	<math.h>
#define TWO_PI (8.0*atan(1.0))

//#define AMBPCD_DEVID 0x4D44 // DeviceID for AMBPCD
//#define AMBPCX_DEVID 0x4D58 // DeviceID for AMBPCX

const U16 PLD_CFG_TAG	= 0x0500; // tag of PLD

void GetOptions();
void DisplayError(S32 status, TCHAR* cmd_str);
S32 PutDataToSrv(BRD_Handle handle, BRD_ServList* srv, int idx);
S32 DacSettings(BRD_Handle hDAC);
S32 FifoOutput(BRD_Handle hDAC, PVOID pSig, ULONG bBufSize);
S32 FifoOutputDMA(BRD_Handle hDAC, int idx, ULONG bBufSize, ULONG bFIFOSize);
void WriteDataFile(int idx, PVOID pBuf, ULONG nNumberOfBytes);

char iniFileName[] = "\\exam_bdac.ini";
// Options
TCHAR g_pldFileName[MAX_PATH];
int g_isPldLoadAlways = 1;

//=************************* main *************************
//=********************************************************
void main( int argc, char *argv[] )
{
	S32		status;

	BRD_displayMode(BRDdm_VISIBLE | BRDdm_CONSOLE); //     :     
	
	S32	DevNum;
	status = BRD_init("brd.ini", &DevNum); //  
//	BRD_Error* pErrInfo = NULL;
//	status = BRD_error(&pErrInfo); //    

	//   LID (   )
	BRD_LidList lidList;
	lidList.item = 10;
	lidList.pLID = new U32[10];
	status = BRD_lidList(lidList.pLID, lidList.item, &lidList.itemReal);

	GetOptions();

	BRD_Info	info;
	info.size = sizeof(info);
	BRD_Handle handle[10];

	for(ULONG iDev = 0; iDev < lidList.itemReal; iDev++)
	{
		BRD_getInfo(lidList.pLID[iDev], &info); //    
		printf("Board: %s, DevID = 0x%x, RevID = 0x%x, Bus = %d, Dev = %d, Slot = %d.\n", 
			info.name, info.boardType >> 16, info.boardType & 0xff, info.bus, info.dev, info.slot);
		//handle[iDev] = BRD_open(lidList.pLID[iDev], BRDopen_EXCLUSIVE, NULL); //     
		handle[iDev] = BRD_open(lidList.pLID[iDev], BRDopen_SHARED, NULL); //     
		if(handle[iDev] > 0)
		{
			BRD_PuList PuList[5];
			U32 ItemReal;
			status = BRD_puList(handle[iDev], PuList, 5, &ItemReal); //    
			if(ItemReal <= 5)
			{
				for(U32 j = 0; j < ItemReal; j++)
				{
					if(PuList[j].puCode == PLD_CFG_TAG)
					{ //    - 
						printf("PU%d: %s, Id = %d, Code = %x, Attr = %x.\n",
										j, PuList[j].puDescription, PuList[j].puId, PuList[j].puCode, PuList[j].puAttr);
						 //   
						U32	PldState;
						status = BRD_puState(handle[iDev], PuList[j].puId, &PldState);
						printf("BRD_puState: PldState = %d\n", PldState);
						if(PuList[j].puId == 0x100)
						{
							if(g_isPldLoadAlways || !PldState)
							{ //   ADM
								printf("BRD_puLoad: loading...\r");
								status = BRD_puLoad(handle[iDev], PuList[j].puId, g_pldFileName, &PldState);
								if(BRD_errcmp(status, BRDerr_OK))
									printf("BRD_puLoad: load is OK\n");
								else
									if(BRD_errcmp(status, BRDerr_PLD_TEST_DATA_ERROR))
										printf("BRD_puLoad: data error starting test\n");
									else
										if(BRD_errcmp(status, BRDerr_PLD_TEST_ADDRESS_ERROR))
											printf("BRD_puLoad: address error starting test\n");
										else
											printf("BRD_puLoad: error loading\n");
							}
						}
					}
				}
			}
			if(BRD_errcmp(status, BRDerr_OK))
			{
				BRD_ServList srvList[10];
				status = BRD_serviceList(handle[iDev], 0, srvList, 10, &ItemReal);
				if(ItemReal <= 10)
				{
					//for(U32 j = 0; j < ItemReal; j++)
					//{
					//	printf("Service %d: %s, Attr = %X.\n",
					//					j, srvList[j].name, srvList[j].attr);
					//}
					U32 iSrv;
					for(iSrv = 0; iSrv < ItemReal; iSrv++)
					{
						if(!strcmp(srvList[iSrv].name, "BASEDAC0"))
							status = PutDataToSrv(handle[iDev], &srvList[iSrv], iDev * ItemReal + iSrv);
					}
				}
			}
			status = BRD_close(handle[iDev]); //   
			printf("BRD_close: OK\n");
		}
	}
	delete lidList.pLID;
	status = BRD_cleanup();
	printf("BRD_cleanup: OK\n");
	printf("Press any key for leaving program...\n");
	_getch();
}

void GetOptions()
{
	TCHAR Buffer[128];
	TCHAR iniFilePath[MAX_PATH];
	GetCurrentDirectory(sizeof(iniFilePath)/sizeof(TCHAR), iniFilePath);
	lstrcat(iniFilePath, iniFileName);
	GetPrivateProfileString("Option", "PldFileName", "ambpcd_v10_adm212x200m.mcs", g_pldFileName, sizeof(g_pldFileName), iniFilePath);
	GetPrivateProfileString("Option", "isPldLoadAlways", "1", Buffer, sizeof(Buffer), iniFilePath);
	g_isPldLoadAlways = atoi(Buffer);
}

void DisplayError(S32 status, TCHAR* cmd_str)
{
	S32	real_status = BRD_errext(status);
	TCHAR msg[255];
	switch(real_status)
	{
	case BRDerr_OK:
		sprintf_s(msg, "%s: BRDerr_OK\n", cmd_str);
		break;
	case BRDerr_BAD_MODE:
		sprintf_s(msg, "%s: BRDerr_BAD_MODE\n", cmd_str);
		break;
	case BRDerr_INSUFFICIENT_SERVICES:
		sprintf_s(msg, "%s: BRDerr_INSUFFICIENT_SERVICES\n", cmd_str);
		break;
	default:
		sprintf_s(msg, "%s: Unknown error\n", cmd_str);
		break;
	}
	printf(msg);
}

S32 PutDataToSrv(BRD_Handle handle, BRD_ServList* srv, int idx)
{
	S32		status;
	U32 mode = BRDcapt_EXCLUSIVE;
	BRD_Handle hDAC = BRD_capture(handle, 0, &mode, srv->name, 10000);
	if(mode == BRDcapt_EXCLUSIVE) printf("DAC Capture mode: EXCLUSIVE\n");
	if(mode == BRDcapt_SPY)	printf("DAC Capture mode: SPY\n");
	if(hDAC > 0)
	{
		ULONG master = BRDims_SINGLE; //    () 
		status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETMASTER, &master);

//		ULONG chan_mask = 3; //    
		ULONG chan_mask = 1; //  0- 
		status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETCHANMASK, &chan_mask);
		int num_chan = (chan_mask & 0x1) + ((chan_mask >> 1) & 0x1);

		ULONG cycling = 0; //        
		status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETCYCLMODE, &cycling);
		if(BRD_errcmp(status, BRDerr_OK))
			printf("BRDctrl_DAC_SETCYCLMODE: cycling = %d\n", cycling);
		else
			DisplayError(status, "BRDctrl_DAC_SETCYCLMODE");

		status = DacSettings(hDAC); //   

		BRD_BaseDacCfg dac_cfg;
		status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_GETCFG, &dac_cfg);
//		ULONG bBufSize = dac_cfg.FifoSize; //      
		ULONG bBufSize = (chan_mask == 0x3) ? dac_cfg.FifoSize : (dac_cfg.FifoSize >> 1); //      
		ULONG samplesOfChannel = bBufSize / num_chan / sizeof(short); //    

		//     FIFO  
		PVOID pSig = new char[bBufSize];
		int samplesCnt = bBufSize / sizeof(short);
		short* sPtr = (short*)pSig;
		for(int i = 0; i < samplesCnt; i++)
//	      sPtr[i] = (short)(-30000.0 + (30000.0/(double)samplesCnt)*(double)i);
		  sPtr[i] = (short)floor((30000.0 * sin(TWO_PI * i / (double)samplesCnt)) + 0.5);

//		WriteDataFile(idx, pSig, bBufSize);

		status = FifoOutput(hDAC, pSig, bBufSize - 8); //   

		delete pSig;

		bBufSize = 4096 * 1024; // 4 MB
		//    
		status = FifoOutputDMA(hDAC, idx, bBufSize, dac_cfg.FifoSize); //   

		status = BRD_release(hDAC, 0);
	}
	return status;
}

//   
S32 DacSettings(BRD_Handle hDAC)
{
	S32		status;

	double rate = 1000000.; // 1 MHz; 
//	double rate = 20000000.; // 20 MHz; 
	//             
	BRD_SyncMode sync_mode;
	sync_mode.clkSrc = BRDclks_BASEGEN0;
	sync_mode.clkValue = 60000000.; // 60 MHz
//	sync_mode.clkSrc = BRDclks_BASEGEN1;
//	sync_mode.clkValue = 50000000.; // 50 MHz
	sync_mode.rate = rate;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETSYNCMODE, &sync_mode);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_SETSYNCMODE: clock = %f, rate = %f\n", sync_mode.clkValue, sync_mode.rate);
	else
		DisplayError(status, "BRDctrl_DAC_SETSYNCMODE");
/*
	//       
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETRATE, &rate);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_SETRATE: rate = %f\n", rate);
	else
		DisplayError(status, "BRDctrl_DAC_SETRATE");

	//         
	BRD_ClkMode clk_mode;
	clk_mode.src = BRDclks_BASEGEN0;
	clk_mode.value = 60000000.; // 60 MHz
//	clk_mode.src = BRDclks_BASEGEN1;
//	clk_mode.value = 50000000.; // 50 MHz
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETCLKMODE, &clk_mode);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_SETCLKMODE: source = %d, value = %f\n", clk_mode.src, clk_mode.value);
	else
		DisplayError(status, "BRDctrl_DAC_SETCLKMODE");
*/
	return status;
}

//         FIFO
S32 FifoOutput(BRD_Handle hDAC, PVOID pSig, ULONG bBufSize)
{
	printf("\nCycling output into DAC (FIFO)\n");

	S32		status;
	ULONG Status = 0;
	ULONG Enable = 0;

//	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFOSTATUS, &Status);

	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_ENABLE: Enable = %d\n", Enable);
	else
		DisplayError(status, "BRDctrl_DAC_ENABLE");

	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_FIFORESET: OK\n");
	else
		DisplayError(status, "BRDctrl_DAC_FIFORESET");

	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFOSTATUS, &Status);

	BRD_DataBuf data_buf;
	data_buf.pData = pSig;
	data_buf.size = bBufSize;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_PUTDATA, &data_buf);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_PUTDATA: OK\n");
	else
		DisplayError(status, "BRDctrl_DAC_PUTDATA");

	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFOSTATUS, &Status);

	ULONG cycling = 1; //  
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETCYCLMODE, &cycling);

	Enable = 1;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_ENABLE: Enable = %d\n", Enable);
	else
		DisplayError(status, "BRDctrl_DAC_ENABLE");

    printf("Hit any key to abort...\n");
	_getch();

	Enable = 0;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_DAC_ENABLE: Enable = %d\n", Enable);
	else
		DisplayError(status, "BRDctrl_DAC_ENABLE");

	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFOSTATUS, &Status);

	cycling = 0; //   
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_SETCYCLMODE, &cycling);

	return status;
}

//       -   
S32 FifoOutputDMA(BRD_Handle hDAC, int idx, ULONG bBufSize, ULONG bFIFOSize)
{
//	printf("\nOne time output into DAC (DMA)\n");
	printf("\nCycling output into DAC (DMA)\n");
	S32		status;
	ULONG Status = 0;

	ULONG Enable = 0;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFORESET, NULL); //  FIFO 

	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_FIFOSTATUS, &Status);

	BRDctrl_StreamCBufAlloc buf_dscr;
	buf_dscr.dir = BRDstrm_DIR_OUT;
//	buf_dscr.isCont = 0; //      
	buf_dscr.isCont = 1; //      
	buf_dscr.blkNum = 1;
	buf_dscr.blkSize = bBufSize;
	buf_dscr.ppBlk = new PVOID[buf_dscr.blkNum];
	status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_ALLOC, &buf_dscr);

	int samplesCnt = bFIFOSize / sizeof(short);
	int cnt = bBufSize / bFIFOSize;
	for(int j = 0; j < cnt; j++) 
	{
		short* sPtr = (short*)buf_dscr.ppBlk[0] + j * samplesCnt;
		for(int i = 0; i < samplesCnt; i++) 
//	      sPtr[i] = (short)(-30000.0 + (30000.0/(double)samplesCnt)*(double)i);
		  sPtr[i] = (short)floor((30000.0 * sin(TWO_PI * i / (double)samplesCnt)) + 0.5);
	}
//	WriteDataFile(idx, buf_dscr.ppBlk[0], bBufSize);

	//     
	ULONG tetrad;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_GETSRCSTREAM, &tetrad); //     
	status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_SETSRC, &tetrad);

	BRDctrl_StreamCBufStart start_pars;
//	start_pars.isCycle = 0; //   
	start_pars.isCycle = 1; //  
	status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_START, &start_pars); //  
	Enable = 1;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   
	int fl_break = 0;
    printf("Hit any key to abort...\n");
	while(!fl_break & !_kbhit())
	{
		ULONG msTimeout = 20000; //       20 .
		status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_WAITBLOCK, &msTimeout);
		if(BRD_errcmp(status, BRDerr_WAIT_TIMEOUT))
		{	//    -,  
			status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_STOP, NULL);
			DisplayError(status, "TIME-OUT");
			fl_break = 1;
		}
	}
	if(!fl_break)
		_getch();

	Enable = 0;
	status = BRD_ctrl(hDAC, 0, BRDctrl_DAC_ENABLE, &Enable); //   

//	WriteDataFile(idx, buf_dscr.ppBlk[0], bBufSize);

	status = BRD_ctrl(hDAC, 0, BRDctrl_STREAM_CBUF_FREE, &buf_dscr);
	delete[] buf_dscr.ppBlk;

	return status;
}

void WriteDataFile(int idx, PVOID pBuf, ULONG nNumberOfBytes)
{
	TCHAR fileName[64];
	sprintf_s(fileName, "data_%d.bin", idx);
	HANDLE hfile = CreateFile(	fileName,
								GENERIC_WRITE,
								FILE_SHARE_WRITE,
								NULL,
								CREATE_ALWAYS,
								FILE_ATTRIBUTE_NORMAL,
								NULL);
	if(hfile == INVALID_HANDLE_VALUE)
	{
		printf("Create file error\n");
		return;
	}
	ULONG nNumberOfWriteBytes;

	BOOL bResult = WriteFile(hfile, pBuf, nNumberOfBytes, &nNumberOfWriteBytes, NULL);
	CloseHandle(hfile);
	if(bResult != TRUE)
		printf("Write file error\n");
}
//
// End of file
//
